#include <stdio.h>
#include <string.h>
#include <stdlib.h>


// La estructura de una persona

#include "pers.h"

// typedef struct {
//   char rut[12];
//   int edad;
//   char nom[30];
// }  Persona;

// lee la i-esima persona en archPers

void leerPers(FILE *archPers, Persona *pers, int i) {
  if (fseek(archPers, i*sizeof(Persona), SEEK_SET)!=0) {
    perror("leerPers");
    exit(1);
  }
  if (fread(pers, sizeof(Persona), 1, archPers)!=1) {
    perror("leerPers");
    exit(1);
  }
}

// (re)escribe la i-esima persona en archPers
void escrPers(FILE *archPers, Persona *pers, int i) {
  if (fseek(archPers, i*sizeof(Persona), SEEK_SET)!=0) {
    perror("escrPers");
    exit(1);
  }
  if (fwrite(pers, sizeof(Persona), 1, archPers)!=1) {
    perror("escrPers");
    exit(1);
  }
}

void swapPers(void *ptr, int i, int j) {
  FILE *archPers = ptr;
  Persona persI, persJ;
  leerPers(archPers, &persI, i);
  leerPers(archPers, &persJ, j);
  escrPers(archPers, &persI, j);
  escrPers(archPers, &persJ, i);
}

int cmpPers(void *ptr, int i, int j) {
  FILE *archPers = ptr;
  Persona persI, persJ;
  leerPers(archPers, &persI, i);
  leerPers(archPers, &persJ, j);
  return strcmp(persI.nom, persJ.nom);
}

// Ordenamiento generico con quicksort

typedef int (*Comparator)(void *ptr, int i, int j);
typedef void (*Swapper)(void *ptr, int i, int j);

void sort(void *ptr, int left, int right,
          Comparator compare, Swapper swap) {
  int i, last;

  if (left>=right)
    return;

  (*swap)(ptr, left, (left+right)/2);
  last= left;

  /*
    +--+-----------+--------+--------------+
    |  |///////////|\\\\\\\\|              |
    +--+-----------+--------+--------------+
    left        last         i         right
  */

  for (i= left+1; i<=right; ++i)
    if ((*compare)(ptr, i, left)<0)
      (*swap)(ptr, ++last, i);
  (*swap)(ptr, left, last);

  sort(ptr, left, last-1, compare, swap);
  sort(ptr, last+1, right, compare, swap);
}

// Ordena un archivo de personas

int main(int argc, char *argv[]) {

  if (argc!=2) {
    fprintf(stderr, "Uso: %s <archivo-personas>\n", argv[0]);
    exit(1);
  }

  char *nomArch= argv[1];

  FILE *archPers= fopen(nomArch, "r+");
  if (archPers==NULL) {
    perror(nomArch);
    exit(1);
  }

  if (fseek(archPers, 0, SEEK_END)!=0) {
    perror(nomArch);
    exit(1);
  }

  int sizeArch= ftell(archPers);
  int numPers= sizeArch / sizeof(Persona);

  sort(archPers, 0, numPers-1, (Comparator)cmpPers, (Swapper)swapPers);

  return 0;
}

